
/*
    hf.c - functions (mostly statistics) for HF contests
    Copyright (C) 2010 Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include "header.h"

int  sw_hf_kbd_func(struct subwin *sw, struct event *ev, int fw){
    return 0;
}

int  sw_hf_mouse_func(struct subwin *sw, struct event *ev, int fw){
    return 0;
}

#define QSOBOUND 4
#define LASTQSO 8

void clip_char(struct subwin *sw, int xrel, int yrel, short c){
    if (xrel < 0) return;
    if (yrel < 0) return;
    if (xrel >= sw->w) return;
    if (yrel >= sw->h) return;
    set_char(sw->x + xrel, sw->y + yrel, c);

}

void clip_printf(struct subwin *sw, int xrel, int yrel, short color, char *fmt, ...){
    va_list l;
    gchar *c, *s;

    va_start(l, fmt);
    s=g_strdup_vprintf(fmt, l);
    va_end(l);
    for (c=s; *c!='\0'; c++){
        clip_char(sw, xrel++, yrel, *c | color);
    }

    g_free(s);

}

#define BANDS 6

void sw_hf_redraw(struct subwin *sw){
    int i,j,x,y;
    int bi;
    struct band *bands[BANDS];
    char s[256], t[256];
    char *c;
    char *hf_modes[] = {"???", "SSB", " CW", "MIX", "MIX", " AM", "FM", "RTY", "STV", "ATV"}; 

    // frames
    for (i=0; i<9;i++) {
        clip_char(sw, 33, i, 0x8000|COL_NORM|179);
        clip_char(sw, 55, i, 0x8000|COL_NORM|179);
    }
    for (i=0; i<33;i++)   clip_char(sw, i, QSOBOUND, 0x8000|COL_NORM|196);
    if (QSOBOUND < sw->h) set_char(sw->x-1, sw->y + QSOBOUND, 0x8000|COL_NORM|195);
    clip_char(sw, 33, QSOBOUND, 0x8000|COL_NORM|180);

    // excs
    if (aband && excdb){
        struct miss_struct ms;
        miss_compute_ms(&ms, aband);
        x = 0;
        y = 0;
        for (i=0; i<ms.ia->len; i++){
            c = (gchar *)g_index_array_index(ms.ia, i);
            if (x + ms.maxlen + 1 > 33) {
                x = 0;
                y++;
                if (y==QSOBOUND) break;
            }  
            clip_printf(sw, x, y, COL_NORM, c);
            x += ms.maxlen + 1;

        }
        g_index_array_free_all(ms.ia);
    }


    // bands prepare
    i = 0;
    memset(bands, 0, sizeof(bands));
    if (ctest){
        for (bi = ctest->bands->len - 1; bi >= 0; bi--){
            struct band *b = (struct band *)g_ptr_array_index(ctest->bands, bi);
            bands[i++] = b;
        }
    }

    // bands stats
    int nqsos=0, nqsop=0, nmult=0;
    if (ctest) 
        clip_printf(sw, 34, 0 , COL_NORM, VTEXT(T_HF_RATE), ctest->qso_per_10 * 6, ctest->qso_per_60);
    else
        clip_printf(sw, 34, 0 , COL_NORM, VTEXT(T_HF_RATE), 0, 0);
    clip_printf(sw, 34, 1 , COL_NORM, VTEXT(T_HF_BAND));
    for (i=0; i<6;i++){
        struct band *b = bands[i];
        if (!b) continue;
        safe_strncpy0(s, b->bandname, sizeof(s));
        c = index(s, ' ');
        if (c>0) *c='\0';
        j = strlen(s);
        if (j > 0 && toupper(s[j-1]) == 'M') s[j-1] = '\0'; 
        clip_printf(sw, 37-strlen(s), i+2, COL_NORM, s);
        nqsos += b->stats->nqsos;
        nqsop += b->stats->nqsop;
        clip_printf(sw, 39, i+2, COL_NORM, "%4d %5d %4d", b->stats->nqsos, b->stats->nqsop, b->stats->nmult);
    }
    clip_printf(sw, 34, i+2 , COL_NORM, VTEXT(T_HF_SUM));
    clip_printf(sw, 39, i+2, COL_NORM, "%4d %5d %4d", nqsos, nqsop, ctest?ctest->allb_nmult:0);

    // qsos
    if (ctest && ctest->allqsos->len > 0){
        y = LASTQSO;
        for (i=ctest->allqsos->len - 1; i >= 0; i--){
            struct qso *q = (struct qso *)g_ptr_array_index(ctest->allqsos, i);
            if (y<=QSOBOUND) continue;
            safe_strncpy0(s, q->band->bandname, sizeof(s));
            c = index(s, ' ');
            if (c>0) *c='\0';
            j = strlen(s);
            if (j > 0 && toupper(s[j-1]) == 'M') s[j-1] = '\0'; 
            clip_printf(sw, 0, y, COL_NORM, s);
            clip_printf(sw, 3, y, COL_NORM, hf_modes[q->mode%10]);
            clip_printf(sw, 7, y, COL_NORM, q->time_str);

            safe_strncpy0(s, q->callsign, 20);
            clip_printf(sw, 12, y, COL_NORM, s);

            strcpy(t, "");
            if (ctest->qsoused > 0) {
                strcat(t, " ");
                strcat(t, q->qsonrr);
            }
            if (*q->exc) {
                strcat(t, " ");
                strcat(t, q->exc);
            }
            if (*q->locator) {
                strcat(t, " ");
                strcat(t, q->locator);
            }
            safe_strncpy0(s, t, 8);
            clip_printf(sw, 23, y, COL_NORM, s);

            sprintf(t, "%d", q->qsop);
            safe_strncpy0(s, t, 2);
            clip_printf(sw, 31, y, COL_NORM, s);
            if (q->new & NEW_MULT) clip_printf(sw, 32, y, COL_NORM, "*");

            if (i==ctest->allqsos->len - 1){ // last QSO
                s[0]=q->time_str[0]; s[1]=q->time_str[1]; s[2]=':';
                s[3]=q->time_str[2]; s[4]=q->time_str[3]; s[5]='\0';
                strcpy(t, "(n/a)");
                if (ctest->lastmultqso){
                    t[0]=ctest->lastmultqso->time_str[0]; t[1]=ctest->lastmultqso->time_str[1]; t[2]=':';
                    t[3]=ctest->lastmultqso->time_str[2]; t[4]=ctest->lastmultqso->time_str[3]; t[5]='\0';
                }
                clip_printf(sw, 57, 0, COL_NORM, VTEXT(T_HF_LAST), s, t);


            }
            
            y--;
        }
    }

    //DXCC
    if (gses->adwi){
        clip_printf(sw, 57, 1, COL_NORM, "%02d %s %s", gses->adwi->waz, gses->adxc, gses->adwi->dxcname);

    }
    clip_char(sw, 60, 8, 0x8000|COL_NORM|179);
    if (gses->asunriseset){
        clip_printf(sw, 61, 8, COL_NORM, "%11s", gses->asunriseset);
	}
    clip_char(sw, 72, 8, 0x8000|COL_NORM|179);
	if (spotdb && spotdb->sfi > 0){
        clip_printf(sw, 73, 8, COL_NORM, "SFI=%d", spotdb->sfi);
	}
}

void sw_hf_check_bounds(struct subwin *sw){
}

void sw_hf_raise(struct subwin *sw){
}

void get_hf_dxc(char *str){
    struct dw_item *dwi;
    static char dxcc[25];
	static char sun[16];

    dwi = get_dw_item_by_call(dw, str);
    if (!dwi) return;

    gses->adwi = dwi;
    
    get_dxcc(dw, dxcc, str);
    gses->adxc = dxcc;
	gses->asunriseset = sun;
	sunrisesetstr(sun, dwi->latitude, dwi->longitude);

}

